<div class="section">
    <h2 class="section-title">File Size Distribution</h2>
    <div class="chart-container full-width" style="min-height: 320px;">
        <canvas id="dynamic-size-histogram-chart"></canvas>
    </div>
</div>